local Aloft = Aloft
if not Aloft then return end

-----------------------------------------------------------------------------

local AloftStateIcon = Aloft:NewModule("StateIcon", Aloft, "AceEvent-3.0", "AceTimer-3.0")

-----------------------------------------------------------------------------

AloftStateIcon.namespace = "stateIcon"
AloftStateIcon.defaults =
{
	profile =
	{
		enable			= true,
		alpha			= 1.0,
		reverse			= false,
		point			= "CENTER",
		relativeToPoint	= "RIGHT",
		offsetX			= -4,
		offsetY			= -1,
		height			= 34,
		width			= 46,
	},
}

-----------------------------------------------------------------------------

-- just a mnemonic, not currently used
local ELITE_NAMEPLATE_ICON = [[Interface\Tooltips\EliteNameplateIcon]]

-----------------------------------------------------------------------------

function AloftStateIcon:UpdateAll()
	for aloftData in Aloft:IterateVisibleNameplates() do
		self:ScheduleTimer(function(aloftData) AloftStateIcon:DoNameplateShow(aloftData) end, 0.25, aloftData) -- a bit more than next frame
	end
end

-----------------------------------------------------------------------------

function AloftStateIcon:OnInitialize()
	if self.db ~= Aloft.AloftDB:GetNamespace(self.namespace, true) then self.db = Aloft.AloftDB:RegisterNamespace(self.namespace, self.defaults) end

	self.initialized = true
end

function AloftStateIcon:OnEnable()
	-- Ace3 now calls OnInitialize only if the addon is available at time of ADDON_LOADED?
	if not self.initialized then self:OnInitialize() end

	self:RegisterMessage("Aloft:SetupFrame", "SetupFrame")
	self:RegisterMessage("Aloft:OnNameplateShow", "OnNameplateShow")
	self:UpdateAll()
end

function AloftStateIcon:OnDisable()
	self:UnregisterAllEvents()
	self:UnregisterAllMessages()
	self:CancelAllTimers()

	for aloftData in Aloft:IterateNameplates() do
		local blizzardStateIconRegion = aloftData.blizzardStateIconRegion

		if blizzardStateIconRegion then
			--[[
			blizzardStateIconRegion:ClearAllPoints()
			blizzardStateIconRegion:SetPoint("CENTER", aloftData.levelTextRegion, "CENTER", 4.70, 1.57)
			blizzardStateIconRegion:SetWidth(46)
			blizzardStateIconRegion:SetHeight(34)
			if blizzardStateIconRegion.ULx then
				blizzardStateIconRegion:SetTexCoord(stateIconRegion.ULx, stateIconRegion.ULy, stateIconRegion.LLx, stateIconRegion.LLy, stateIconRegion.URx, stateIconRegion.URy, stateIconRegion.LRx, stateIconRegion.LRy) -- un-flip/mirror horizontally, restore to original values
			end
			blizzardStateIconRegion:SetDrawLayer("OVERLAY")
			]]

			blizzardStateIconRegion:SetAlpha(1)
		end
	end
end

function AloftStateIcon:SetupFrame(message, aloftData)
	-- self:ScheduleTimer(function(aloftData) AloftStateIcon:DoSetupFrame(aloftData) end, 0.15, aloftData) -- a bit more than next frame
	if aloftData then
		self:DoSetupFrame(aloftData)
	end
end

function AloftStateIcon:DoSetupFrame(aloftData)
	if aloftData then -- apparently we can get here with a nil nameplate
		local layoutFrame = aloftData.layoutFrame
		if not layoutFrame then
			layoutFrame = Aloft:AcquireLayoutFrame(aloftData)
		end
		local stateIconRegion = layoutFrame.stateIconRegion

		if not stateIconRegion then
			stateIconRegion = aloftData:CreateTexture()

			layoutFrame.stateIconRegion = stateIconRegion
		end

		local blizzardStateIconRegion = aloftData.blizzardStateIconRegion
		stateIconRegion:SetTexture(blizzardStateIconRegion:GetTexture())
			
		-- local r, g, b, a = stateIconRegion:GetVertexColor()
		-- local tr, tg, tb = floor(r * 256), floor(g*256), floor(b*256)
		-- ChatFrame7:AddMessage("AloftStateIcon:DoSetupFrame(): " .. ("|cff%02x%02x%02x%s|r"):format(tr, tg, tb, tostring(aloftData.name) .. "/" .. tostring(tr) .. "." .. tostring(tg) .. "." .. tostring(tb)) .. "/" .. tostring(stateIconRegion:IsVisible()) .. "/" .. tostring(stateIconRegion:GetTexture()))

		-- mirror texture if needed
		if not stateIconRegion.ULx then
			stateIconRegion.ULx, stateIconRegion.ULy, stateIconRegion.LLx, stateIconRegion.LLy, stateIconRegion.URx, stateIconRegion.URy, stateIconRegion.LRx, stateIconRegion.LRy = stateIconRegion:GetTexCoord() -- collect the coordinates currently in use
		end
		if self.db.profile.reverse then
			stateIconRegion:SetTexCoord(stateIconRegion.URx, stateIconRegion.URy, stateIconRegion.LRx, stateIconRegion.LRy, stateIconRegion.ULx, stateIconRegion.ULy, stateIconRegion.LLx, stateIconRegion.LLy) -- flip/mirror horizontally
		else
			stateIconRegion:SetTexCoord(stateIconRegion.ULx, stateIconRegion.ULy, stateIconRegion.LLx, stateIconRegion.LLy, stateIconRegion.URx, stateIconRegion.URy, stateIconRegion.LRx, stateIconRegion.LRy) -- un-flip/mirror horizontally, restore to original values
		end

		if not blizzardStateIconRegion:IsVisible() then
			stateIconRegion:Hide()
		else
			stateIconRegion:SetAlpha(1)
			stateIconRegion:Show()
		end

		self:PlaceFrame(stateIconRegion, layoutFrame, self.db.profile) -- this sets alpha to 0 if enable is not true
		stateIconRegion:SetDrawLayer("ARTWORK", 7)
		stateIconRegion:SetBlendMode("BLEND")

		-- adjust after additional frame delay
		-- self:ScheduleTimer(function(aloftData) AloftStateIcon:DoAdjustFrame(aloftData) end, 0.15, aloftData) -- a bit more than next frame
	end
end

function AloftStateIcon:OnNameplateShow(message, aloftData)
	-- self:ScheduleTimer(function(aloftData) AloftStateIcon:DoNameplateShow(aloftData) end, 0.15, aloftData) -- a bit more than next frame
	if aloftData then
		self:DoNameplateShow(aloftData)
	end
end

function AloftStateIcon:DoNameplateShow(aloftData)
	self:DoSetupFrame(aloftData)
end

function AloftStateIcon:DoAdjustFrame(aloftData)
	local data = self.db.profile
	aloftData.stateIconRegion:SetPoint(data.point, aloftData.layoutFrame, data.relativeToPoint, data.offsetX + 3, data.offsetY + 3) -- set slightly off, corrected after additional frame delay

	self:ScheduleTimer(function(aloftData) AloftStateIcon:DoReAdjustFrame(aloftData) end, 0.15, aloftData) -- a bit more than next frame
end

function AloftStateIcon:DoReAdjustFrame(aloftData)
	local data = self.db.profile
	aloftData.stateIconRegion:SetPoint(data.point, aloftData.layoutFrame, data.relativeToPoint, data.offsetX, data.offsetY) -- now correct after additional frame delay
end

-----------------------------------------------------------------------------
